@@ -16,7 +16,6 @@ from .utils import _range
 # to inherit from the tqdm class
 from .std import tqdm as std_tqdm
 
-
 if True:  # pragma: no cover
     # import IPython/Jupyter base widget and display utilities
     IPY = 0
@@ -33,8 +32,7 @@ if True:  # pragma: no cover
         import warnings
         with warnings.catch_warnings():
             warnings.filterwarnings(
-                'ignore',
-                message=".*The `IPython.html` package has been deprecated.*")
+                'ignore', message=".*The `IPython.html` package has been deprecated.*")
             try:
                 import IPython.html.widgets as ipywidgets
             except ImportError:
@@ -57,6 +55,7 @@ if True:  # pragma: no cover
         except ImportError:
             IPY = 0
             IProgress = None
+            HBox = object
 
     try:
         from IPython.display import display  # , clear_output
@@ -69,16 +68,35 @@ if True:  # pragma: no cover
     except ImportError:  # Py2
         from cgi import escape
 
-
 __author__ = {"github.com/": ["lrq3000", "casperdcl", "alexanderkuk"]}
 __all__ = ['tqdm_notebook', 'tnrange', 'tqdm', 'trange']
 
 
+class TqdmHBox(HBox):
+    """`ipywidgets.HBox` with a pretty representation"""
+    def _repr_json_(self, pretty=None):
+        if not hasattr(self, "pbar"):
+            return {}
+        d = dict(self.pbar.format_dict,
+                 bar_format=(self.pbar.format_dict['bar_format']
+                             or "{l_bar}{bar}{r_bar}").replace("<bar/>", "{bar}"))
+        if pretty is not None:
+            d["ascii"] = not pretty
+        return d
+
+    def __repr__(self, pretty=False):
+        if not hasattr(self, "pbar"):
+            return super(TqdmHBox, self).__repr__()
+        return self.pbar.format_meter(**self._repr_json_(pretty))
+
+    def _repr_pretty_(self, pp, *_, **__):
+        pp.text(self.__repr__(True))
+
+
 class tqdm_notebook(std_tqdm):
     """
     Experimental IPython/Jupyter Notebook widget using tqdm!
     """
-
     @staticmethod
     def status_printer(_, total=None, desc=None, ncols=None):
         """
@@ -110,7 +128,7 @@ class tqdm_notebook(std_tqdm):
         rtext = HTML()
         if desc:
             ltext.value = desc
-        container = HBox(children=[ltext, pbar, rtext])
+        container = TqdmHBox(children=[ltext, pbar, rtext])
         # Prepare layout
         if ncols is not None:  # use default style of ipywidgets
             # ncols could be 100, "100px", "100%"
@@ -124,7 +142,6 @@ class tqdm_notebook(std_tqdm):
             container.layout.width = ncols
             container.layout.display = 'inline-flex'
             container.layout.flex_flow = 'row wrap'
-        display(container)
 
         return container
 
@@ -195,6 +212,14 @@ class tqdm_notebook(std_tqdm):
             self.container.children[-2].style.bar_color = bar_color
 
     def __init__(self, *args, **kwargs):
+        """
+        Supports the usual `tqdm.tqdm` parameters as well as those listed below.
+
+        Parameters
+        ----------
+        display  : Whether to call `display(self.container)` immediately
+            [default: True].
+        """
         kwargs = kwargs.copy()
         # Setup default output
         file_kwarg = kwargs.get('file', sys.stderr)
@@ -204,14 +229,14 @@ class tqdm_notebook(std_tqdm):
         # Initialize parent class + avoid printing by using gui=True
         kwargs['gui'] = True
         if 'bar_format' in kwargs:
-            kwargs['bar_format'] = kwargs['bar_format'].replace(
-                '{bar}', '<bar/>')
+            kwargs['bar_format'] = kwargs['bar_format'].replace('{bar}', '<bar/>')
         # convert disable = None to False
         kwargs['disable'] = bool(kwargs.get('disable', False))
         colour = kwargs.pop('colour', None)
+        display_here = kwargs.pop('display', True)
         super(tqdm_notebook, self).__init__(*args, **kwargs)
         if self.disable or not kwargs['gui']:
-            self.sp = lambda *_, **__: None
+            self.disp = lambda *_, **__: None
             return
 
         # Get bar width
@@ -220,9 +245,11 @@ class tqdm_notebook(std_tqdm):
         # Replace with IPython progress bar display (with correct total)
         unit_scale = 1 if self.unit_scale is True else self.unit_scale or 1
         total = self.total * unit_scale if self.total else self.total
-        self.container = self.status_printer(
-            self.fp, total, self.desc, self.ncols)
-        self.sp = self.display
+        self.container = self.status_printer(self.fp, total, self.desc, self.ncols)
+        self.container.pbar = self
+        if display_here:
+            display(self.container)
+        self.disp = self.display
         self.colour = colour
 
         # Print initial bar state
@@ -236,7 +263,7 @@ class tqdm_notebook(std_tqdm):
                 yield obj
         # NB: except ... [ as ...] breaks IPython async KeyboardInterrupt
         except:  # NOQA
-            self.sp(bar_style='danger')
+            self.disp(bar_style='danger')
             raise
         # NB: don't `finally: close()`
         # since this could be a shared bar which the user will `reset()`
@@ -248,7 +275,7 @@ class tqdm_notebook(std_tqdm):
         except:  # NOQA
             # cannot catch KeyboardInterrupt when using manual tqdm
             # as the interrupt will most likely happen on another statement
-            self.sp(bar_style='danger')
+            self.disp(bar_style='danger')
             raise
         # NB: don't `finally: close()`
         # since this could be a shared bar which the user will `reset()`
@@ -258,16 +285,15 @@ class tqdm_notebook(std_tqdm):
         # Try to detect if there was an error or KeyboardInterrupt
         # in manual mode: if n < total, things probably got wrong
         if self.total and self.n < self.total:
-            self.sp(bar_style='danger')
+            self.disp(bar_style='danger')
         else:
             if self.leave:
-                self.sp(bar_style='success')
+                self.disp(bar_style='success')
             else:
-                self.sp(close=True)
+                self.disp(close=True)
 
-    def moveto(self, *_, **__):
-        # void -> avoid extraneous `\n` in IPython output cell
-        return
+    def clear(self, *_, **__):
+        pass
 
     def reset(self, total=None):
         """
